@@ -190,29 +190,6 @@ var wakaUnknownBefore;
  */
 var MozBlobBuilder;
 
-/**
- * Some JS libraries make conditional calls to dynCall_xxx function when
- * callbacks are registered.  For example the exit runtime handler makes
- * conditional calles to dynCall_v and dynCall_vi, but not all wasm binaryies
- * will contain these.
- */
-/**
- * @suppress {duplicate, undefinedVars}
- */
-var dynCall_v;
-/**
- * @suppress {duplicate, undefinedVars}
- */
-var dynCall_vi;
-/**
- * @suppress {duplicate, undefinedVars}
- */
-var dynCall_vii;
-/**
- * @suppress {duplicate, undefinedVars}
- */
-var dynCall_iii;
-
 // Module loaders externs, for AMD etc.
 
 /**
@@ -295,10 +272,6 @@ var devicePixelRatio;
 // Variables that are present in both output runtime .js file/JS lib files, and worker.js, so cannot be minified because
 // the names need to match:
 /** @suppress {duplicate} */
-var threadInfoStruct;
-/** @suppress {duplicate} */
-var selfThreadId;
-/** @suppress {duplicate} */
 var noExitRuntime;
 
 /** @type {?AudioWorklet} */
