@@ -18,12 +18,13 @@
 /*global typeDependencies, flushPendingDeletes, getTypeName, getBasestPointer, throwBindingError, UnboundTypeError, _embind_repr, registeredInstances, registeredTypes, getShiftFromSize*/
 /*global ensureOverloadTable, embind__requireFunction, awaitingDependencies, makeLegalFunctionName, embind_charCodes:true, registerType, createNamedFunction, RegisteredPointer, throwInternalError*/
 /*global simpleReadValueFromPointer, floatReadValueFromPointer, integerReadValueFromPointer, enumReadValueFromPointer, replacePublicSymbol, craftInvokerFunction, tupleRegistrations*/
+/*global finalizationGroup, attachFinalizer, detachFinalizer, releaseClassHandle, runDestructor*/
 /*global ClassHandle, makeClassHandle, structRegistrations, whenDependentTypesAreResolved, BindingError, deletionQueue, delayFunction:true, upcastPointer*/
 /*global exposePublicSymbol, heap32VectorToArray, new_, RegisteredPointer_getPointee, RegisteredPointer_destructor, RegisteredPointer_deleteObject, char_0, char_9*/
 /*global getInheritedInstanceCount, getLiveInheritedInstances, setDelayFunction, InternalError, runDestructors*/
 /*global requireRegisteredType, unregisterInheritedInstance, registerInheritedInstance, PureVirtualError, throwUnboundTypeError*/
 /*global assert, validateThis, downcastPointer, registeredPointers, RegisteredClass, getInheritedInstance, ClassHandle_isAliasOf, ClassHandle_clone, ClassHandle_isDeleted, ClassHandle_deleteLater*/
-/*global throwInstanceAlreadyDeleted, runDestructor, shallowCopyInternalPointer*/
+/*global throwInstanceAlreadyDeleted, shallowCopyInternalPointer*/
 /*global RegisteredPointer_fromWireType, constNoSmartPtrRawPointerToWireType, nonConstNoSmartPtrRawPointerToWireType, genericPointerToWireType*/
 
 var LibraryEmbind = {
@@ -125,6 +126,7 @@ var LibraryEmbind = {
    the appropriate overload to call from an function overload table. This selector function is only used if multiple overloads are
    actually registered, since it carries a slight performance penalty. */
   $exposePublicSymbol__deps: ['$ensureOverloadTable', '$throwBindingError'],
+  $exposePublicSymbol__docs: '/** @param {number=} numArguments */',
   $exposePublicSymbol: function(name, value, numArguments) {
     if (Module.hasOwnProperty(name)) {
         if (undefined === numArguments || (undefined !== Module[name].overloadTable && undefined !== Module[name].overloadTable[numArguments])) {
@@ -149,6 +151,7 @@ var LibraryEmbind = {
   },
 
   $replacePublicSymbol__deps: ['$throwInternalError'],
+  $replacePublicSymbol__docs: '/** @param {number=} numArguments */',
   $replacePublicSymbol: function(name, value, numArguments) {
     if (!Module.hasOwnProperty(name)) {
         throwInternalError('Replacing nonexistant public symbol');
@@ -297,6 +300,7 @@ var LibraryEmbind = {
     '$awaitingDependencies', '$registeredTypes',
     '$typeDependencies', '$throwBindingError',
     '$whenDependentTypesAreResolved'],
+  $registerType__docs: '/** @param {Object=} options */',
   $registerType: function(rawType, registeredInstance, options) {
     options = options || {};
 
@@ -392,7 +396,7 @@ var LibraryEmbind = {
     return ret;
   },
 
-  $getTypeName__deps: ['free', '$readLatin1String'],
+  $getTypeName__deps: ['$readLatin1String'],
   $getTypeName: function(type) {
     var ptr = ___getTypeName(type);
     var rv = readLatin1String(ptr);
@@ -612,7 +616,7 @@ var LibraryEmbind = {
   },
 
   _embind_register_std_string__deps: [
-    'free', 'malloc', '$readLatin1String', '$registerType',
+    '$readLatin1String', '$registerType',
     '$simpleReadValueFromPointer', '$throwBindingError'],
   _embind_register_std_string: function(rawType, name) {
     name = readLatin1String(name);
@@ -630,36 +634,23 @@ var LibraryEmbind = {
             var length = HEAPU32[value >> 2];
 
             var str;
-            if(stdStringIsUTF8) {
-                //ensure null termination at one-past-end byte if not present yet
-                var endChar = HEAPU8[value + 4 + length];
-                var endCharSwap = 0;
-                if(endChar != 0)
-                {
-                  endCharSwap = endChar;
-                  HEAPU8[value + 4 + length] = 0;
-                }
-
+            if (stdStringIsUTF8) {
                 var decodeStartPtr = value + 4;
-                //looping here to support possible embedded '0' bytes
+                // Looping here to support possible embedded '0' bytes
                 for (var i = 0; i <= length; ++i) {
-                  var currentBytePtr = value + 4 + i;
-                  if(HEAPU8[currentBytePtr] == 0)
-                  {
-                    var stringSegment = UTF8ToString(decodeStartPtr);
-                    if(str === undefined)
-                      str = stringSegment;
-                    else
-                    {
-                      str += String.fromCharCode(0);
-                      str += stringSegment;
+                    var currentBytePtr = value + 4 + i;
+                    if (i == length || HEAPU8[currentBytePtr] == 0) {
+                        var maxRead = currentBytePtr - decodeStartPtr;
+                        var stringSegment = UTF8ToString(decodeStartPtr, maxRead);
+                        if (str === undefined) {
+                            str = stringSegment;
+                        } else {
+                            str += String.fromCharCode(0);
+                            str += stringSegment;
+                        }
+                        decodeStartPtr = currentBytePtr + 1;
                     }
-                    decodeStartPtr = currentBytePtr + 1;
-                  }
                 }
-
-                if(endCharSwap != 0)
-                  HEAPU8[value + 4 + length] = endCharSwap;
             } else {
                 var a = new Array(length);
                 for (var i = 0; i < length; ++i) {
@@ -669,14 +660,14 @@ var LibraryEmbind = {
             }
 
             _free(value);
-            
+
             return str;
         },
         'toWireType': function(destructors, value) {
             if (value instanceof ArrayBuffer) {
                 value = new Uint8Array(value);
             }
-            
+
             var getLength;
             var valueIsOfTypeString = (typeof value === 'string');
 
@@ -688,16 +679,18 @@ var LibraryEmbind = {
             } else {
                 getLength = function() {return value.length;};
             }
-            
+
             // assumes 4-byte alignment
             var length = getLength();
             var ptr = _malloc(4 + length + 1);
+#if CAN_ADDRESS_2GB
+            ptr >>>= 0;
+#endif
             HEAPU32[ptr >> 2] = length;
-
             if (stdStringIsUTF8 && valueIsOfTypeString) {
                 stringToUTF8(value, ptr + 4, length + 1);
             } else {
-                if(valueIsOfTypeString) {
+                if (valueIsOfTypeString) {
                     for (var i = 0; i < length; ++i) {
                         var charCode = value.charCodeAt(i);
                         if (charCode > 255) {
@@ -725,42 +718,68 @@ var LibraryEmbind = {
   },
 
   _embind_register_std_wstring__deps: [
-    'free', 'malloc', '$readLatin1String', '$registerType',
+    '$readLatin1String', '$registerType',
     '$simpleReadValueFromPointer'],
   _embind_register_std_wstring: function(rawType, charSize, name) {
-    // nb. do not cache HEAPU16 and HEAPU32, they may be destroyed by enlargeMemory().
     name = readLatin1String(name);
-    var getHeap, shift;
+    var decodeString, encodeString, getHeap, lengthBytesUTF, shift;
     if (charSize === 2) {
+        decodeString = UTF16ToString;
+        encodeString = stringToUTF16;
+        lengthBytesUTF = lengthBytesUTF16;
         getHeap = function() { return HEAPU16; };
         shift = 1;
     } else if (charSize === 4) {
+        decodeString = UTF32ToString;
+        encodeString = stringToUTF32;
+        lengthBytesUTF = lengthBytesUTF32;
         getHeap = function() { return HEAPU32; };
         shift = 2;
     }
     registerType(rawType, {
         name: name,
         'fromWireType': function(value) {
-            var HEAP = getHeap();
+            // Code mostly taken from _embind_register_std_string fromWireType
             var length = HEAPU32[value >> 2];
-            var a = new Array(length);
-            var start = (value + 4) >> shift;
-            for (var i = 0; i < length; ++i) {
-                a[i] = String.fromCharCode(HEAP[start + i]);
+            var HEAP = getHeap();
+            var str;
+
+            var decodeStartPtr = value + 4;
+            // Looping here to support possible embedded '0' bytes
+            for (var i = 0; i <= length; ++i) {
+                var currentBytePtr = value + 4 + i * charSize;
+                if (i == length || HEAP[currentBytePtr >> shift] == 0) {
+                    var maxReadBytes = currentBytePtr - decodeStartPtr;
+                    var stringSegment = decodeString(decodeStartPtr, maxReadBytes);
+                    if (str === undefined) {
+                        str = stringSegment;
+                    } else {
+                        str += String.fromCharCode(0);
+                        str += stringSegment;
+                    }
+                    decodeStartPtr = currentBytePtr + charSize;
+                }
             }
+
             _free(value);
-            return a.join('');
+
+            return str;
         },
         'toWireType': function(destructors, value) {
-            // assumes 4-byte alignment
-            var HEAP = getHeap();
-            var length = value.length;
-            var ptr = _malloc(4 + length * charSize);
-            HEAPU32[ptr >> 2] = length;
-            var start = (ptr + 4) >> shift;
-            for (var i = 0; i < length; ++i) {
-                HEAP[start + i] = value.charCodeAt(i);
+            if (!(typeof value === 'string')) {
+                throwBindingError('Cannot pass non-string to C++ string type ' + name);
             }
+
+            // assumes 4-byte alignment
+            var length = lengthBytesUTF(value);
+            var ptr = _malloc(4 + length + charSize);
+#if CAN_ADDRESS_2GB
+            ptr >>>= 0;
+#endif
+            HEAPU32[ptr >> 2] = length >> shift;
+
+            encodeString(value, ptr + 4, length + charSize);
+
             if (destructors !== null) {
                 destructors.push(_free, ptr);
             }
@@ -816,7 +835,7 @@ var LibraryEmbind = {
         var heap = HEAPU32;
         var size = heap[handle]; // in elements
         var data = heap[handle + 1]; // byte offset into emscripten heap
-        return new TA(heap['buffer'], data, size);
+        return new TA(buffer, data, size);
     }
 
     name = readLatin1String(name);
@@ -914,27 +933,33 @@ var LibraryEmbind = {
     var returns = (argTypes[0].name !== "void");
 
 #if DYNAMIC_EXECUTION == 0
-    var argsWired = new Array(argCount - 2);
+    var expectedArgCount = argCount - 2;
+    var argsWired = new Array(expectedArgCount);
+    var invokerFuncArgs = [];
+    var destructors = [];
     return function() {
-      if (arguments.length !== argCount - 2) {
-        throwBindingError('function ' + humanName + ' called with ' + arguments.length + ' arguments, expected ' + (argCount - 2) + ' args!');
+      if (arguments.length !== expectedArgCount) {
+        throwBindingError('function ' + humanName + ' called with ' +
+          arguments.length + ' arguments, expected ' + expectedArgCount +
+          ' args!');
       }
 #if EMSCRIPTEN_TRACING
       Module.emscripten_trace_enter_context('embind::' + humanName);
 #endif
-      var destructors = needsDestructorStack ? [] : null;
+      destructors.length = 0;
       var thisWired;
+      invokerFuncArgs.length = isClassMethodFunc ? 2 : 1;
+      invokerFuncArgs[0] = cppTargetFunc;
       if (isClassMethodFunc) {
         thisWired = argTypes[1].toWireType(destructors, this);
+        invokerFuncArgs[1] = thisWired;
       }
-      for (var i = 0; i < argCount - 2; ++i) {
+      for (var i = 0; i < expectedArgCount; ++i) {
         argsWired[i] = argTypes[i + 2].toWireType(destructors, arguments[i]);
+        invokerFuncArgs.push(argsWired[i]);
       }
 
-      var invokerFuncArgs = isClassMethodFunc ?
-          [cppTargetFunc, thisWired] : [cppTargetFunc];
-
-      var rv = cppInvokerFunc.apply(null, invokerFuncArgs.concat(argsWired));
+      var rv = cppInvokerFunc.apply(null, invokerFuncArgs);
 
       if (needsDestructorStack) {
         runDestructors(destructors);
@@ -1037,64 +1062,28 @@ var LibraryEmbind = {
 #endif
   },
 
-  $embind__requireFunction__deps: ['$readLatin1String', '$throwBindingError'],
+  $embind__requireFunction__deps: ['$readLatin1String', '$throwBindingError'
+#if DYNCALLS || !WASM_BIGINT
+    , '$getDynCaller'
+#endif
+  ],
   $embind__requireFunction: function(signature, rawFunction) {
     signature = readLatin1String(signature);
 
-    function makeDynCaller(dynCall) {
-#if DYNAMIC_EXECUTION == 0
-      return function() {
-          var args = new Array(arguments.length + 1);
-          args[0] = rawFunction;
-          for (var i = 0; i < arguments.length; i++) {
-            args[i + 1] = arguments[i];
-          }
-          return dynCall.apply(null, args);
-      };
+    function makeDynCaller() {
+#if DYNCALLS
+      return getDynCaller(signature, rawFunction);
 #else
-        var args = [];
-        for (var i = 1; i < signature.length; ++i) {
-            args.push('a' + i);
-        }
-
-        var name = 'dynCall_' + signature + '_' + rawFunction;
-        var body = 'return function ' + name + '(' + args.join(', ') + ') {\n';
-        body    += '    return dynCall(rawFunction' + (args.length ? ', ' : '') + args.join(', ') + ');\n';
-        body    += '};\n';
-
-        return (new Function('dynCall', 'rawFunction', body))(dynCall, rawFunction);
+#if !WASM_BIGINT
+      if (signature.indexOf('j') != -1) {
+        return getDynCaller(signature, rawFunction);
+      }
+#endif
+      return wasmTable.get(rawFunction);
 #endif
     }
 
-    var fp;
-    if (Module['FUNCTION_TABLE_' + signature] !== undefined) {
-        fp = Module['FUNCTION_TABLE_' + signature][rawFunction];
-    } else if (typeof FUNCTION_TABLE !== "undefined") {
-        fp = FUNCTION_TABLE[rawFunction];
-    } else {
-        // asm.js does not give direct access to the function tables,
-        // and thus we must go through the dynCall interface which allows
-        // calling into a signature's function table by pointer value.
-        //
-        // https://github.com/dherman/asm.js/issues/83
-        //
-        // This has three main penalties:
-        // - dynCall is another function call in the path from JavaScript to C++.
-        // - JITs may not predict through the function table indirection at runtime.
-        var dc = Module['dynCall_' + signature];
-        if (dc === undefined) {
-            // We will always enter this branch if the signature
-            // contains 'f' and PRECISE_F32 is not enabled.
-            //
-            // Try again, replacing 'f' with 'd'.
-            dc = Module['dynCall_' + signature.replace(/f/g, 'd')];
-            if (dc === undefined) {
-                throwBindingError("No dynCall invoker for signature: " + signature);
-            }
-        }
-        fp = makeDynCaller(dc);
-    }
-
+    var fp = makeDynCaller();
     if (typeof fp !== "function") {
         throwBindingError("unknown function pointer with signature " + signature + ": " + rawFunction);
     }
@@ -1229,6 +1218,7 @@ var LibraryEmbind = {
 
   $structRegistrations: {},
 
+  _embind_register_value_object__sig: 'viiiiii',
   _embind_register_value_object__deps: [
     '$structRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_object: function(
@@ -1247,6 +1237,7 @@ var LibraryEmbind = {
     };
   },
 
+  _embind_register_value_object_field__sig: 'viiiiiiiiii',
   _embind_register_value_object_field__deps: [
     '$structRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_object_field: function(
@@ -1272,6 +1263,7 @@ var LibraryEmbind = {
     });
   },
 
+  _embind_finalize_value_object__sig: 'ii',
   _embind_finalize_value_object__deps: [
     '$structRegistrations', '$runDestructors',
     '$simpleReadValueFromPointer', '$whenDependentTypesAreResolved'],
@@ -1322,7 +1314,7 @@ var LibraryEmbind = {
                 // assume all fields are present without checking.
                 for (var fieldName in fields) {
                     if (!(fieldName in o)) {
-                        throw new TypeError('Missing field');
+                        throw new TypeError('Missing field:  "' + fieldName + '"');
                     }
                 }
                 var ptr = rawConstructor();
@@ -1478,6 +1470,14 @@ var LibraryEmbind = {
     RegisteredPointer.prototype['fromWireType'] = RegisteredPointer_fromWireType;
   },
 
+  $RegisteredPointer__docs: `/** @constructor
+    @param {*=} pointeeType,
+    @param {*=} sharingPolicy,
+    @param {*=} rawGetPointee,
+    @param {*=} rawConstructor,
+    @param {*=} rawShare,
+    @param {*=} rawDestructor,
+     */`,
   $RegisteredPointer__deps: [
     '$constNoSmartPtrRawPointerToWireType', '$genericPointerToWireType',
     '$nonConstNoSmartPtrRawPointerToWireType', '$init_RegisteredPointer'],
@@ -1626,7 +1626,60 @@ var LibraryEmbind = {
     }
   },
 
-  $makeClassHandle__deps: ['$throwInternalError'],
+  $runDestructor: function($$) {
+    if ($$.smartPtr) {
+        $$.smartPtrType.rawDestructor($$.smartPtr);
+    } else {
+        $$.ptrType.registeredClass.rawDestructor($$.ptr);
+    }
+  },
+
+  $releaseClassHandle__deps: ['$runDestructor'],
+  $releaseClassHandle: function($$) {
+    $$.count.value -= 1;
+    var toDelete = 0 === $$.count.value;
+    if (toDelete) {
+        runDestructor($$);
+    }
+  },
+
+  $finalizationGroup: false,
+
+  $detachFinalizer_deps: ['$finalizationGroup'],
+  $detachFinalizer: function(handle) {},
+
+  $attachFinalizer__deps: ['$finalizationGroup', '$detachFinalizer',
+                           '$releaseClassHandle'],
+  $attachFinalizer: function(handle) {
+    if ('undefined' === typeof FinalizationGroup) {
+        attachFinalizer = function (handle) { return handle; };
+        return handle;
+    }
+    // If the running environment has a FinalizationGroup (see
+    // https://github.com/tc39/proposal-weakrefs), then attach finalizers
+    // for class handles.  We check for the presence of FinalizationGroup
+    // at run-time, not build-time.
+    finalizationGroup = new FinalizationGroup(function (iter) {
+        for (var result = iter.next(); !result.done; result = iter.next()) {
+            var $$ = result.value;
+            if (!$$.ptr) {
+                console.warn('object already deleted: ' + $$.ptr);
+            } else {
+                releaseClassHandle($$);
+            }
+        }
+    });
+    attachFinalizer = function(handle) {
+        finalizationGroup.register(handle, handle.$$, handle.$$);
+        return handle;
+    };
+    detachFinalizer = function(handle) {
+        finalizationGroup.unregister(handle.$$);
+    };
+    return attachFinalizer(handle);
+  },
+
+  $makeClassHandle__deps: ['$throwInternalError', '$attachFinalizer'],
   $makeClassHandle: function(prototype, record) {
     if (!record.ptrType || !record.ptr) {
         throwInternalError('makeClassHandle requires ptr and ptrType');
@@ -1637,11 +1690,11 @@ var LibraryEmbind = {
         throwInternalError('Both smartPtrType and smartPtr must be specified');
     }
     record.count = { value: 1 };
-    return Object.create(prototype, {
+    return attachFinalizer(Object.create(prototype, {
         $$: {
             value: record,
         },
-    });
+    }));
   },
 
   $init_ClassHandle__deps: [
@@ -1695,7 +1748,7 @@ var LibraryEmbind = {
     throwBindingError(getInstanceTypeName(obj) + ' instance already deleted');
   },
 
-  $ClassHandle_clone__deps: ['$shallowCopyInternalPointer', '$throwInstanceAlreadyDeleted'],
+  $ClassHandle_clone__deps: ['$shallowCopyInternalPointer', '$throwInstanceAlreadyDeleted', '$attachFinalizer'],
   $ClassHandle_clone: function() {
     if (!this.$$.ptr) {
         throwInstanceAlreadyDeleted(this);
@@ -1705,11 +1758,11 @@ var LibraryEmbind = {
         this.$$.count.value += 1;
         return this;
     } else {
-        var clone = Object.create(Object.getPrototypeOf(this), {
+        var clone = attachFinalizer(Object.create(Object.getPrototypeOf(this), {
             $$: {
                 value: shallowCopyInternalPointer(this.$$),
             }
-        });
+        }));
 
         clone.$$.count.value += 1;
         clone.$$.deleteScheduled = false;
@@ -1717,17 +1770,8 @@ var LibraryEmbind = {
     }
   },
 
-  $runDestructor: function(handle) {
-    var $$ = handle.$$;
-    if ($$.smartPtr) {
-        $$.smartPtrType.rawDestructor($$.smartPtr);
-    } else {
-        $$.ptrType.registeredClass.rawDestructor($$.ptr);
-    }
-  },
-
-  $ClassHandle_delete__deps: [
-    '$runDestructor', '$throwBindingError', '$throwInstanceAlreadyDeleted'],
+  $ClassHandle_delete__deps: ['$releaseClassHandle', '$throwBindingError',
+                              '$detachFinalizer', '$throwInstanceAlreadyDeleted'],
   $ClassHandle_delete: function() {
     if (!this.$$.ptr) {
         throwInstanceAlreadyDeleted(this);
@@ -1737,11 +1781,9 @@ var LibraryEmbind = {
         throwBindingError('Object already scheduled for deletion');
     }
 
-    this.$$.count.value -= 1;
-    var toDelete = 0 === this.$$.count.value;
-    if (toDelete) {
-        runDestructor(this);
-    }
+    detachFinalizer(this);
+    releaseClassHandle(this.$$);
+
     if (!this.$$.preservePointerOnDelete) {
         this.$$.smartPtr = undefined;
         this.$$.ptr = undefined;
@@ -1790,6 +1832,7 @@ var LibraryEmbind = {
     }
   },
 
+  $RegisteredClass__docs: '/** @constructor */',
   $RegisteredClass: function(
     name,
     constructor,
@@ -1950,8 +1993,11 @@ var LibraryEmbind = {
     invoker,
     rawConstructor
   ) {
+    assert(argCount > 0);
     var rawArgTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     invoker = embind__requireFunction(invokerSignature, invoker);
+    var args = [rawConstructor];
+    var destructors = [];
 
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
         classType = classType[0];
@@ -1972,9 +2018,8 @@ var LibraryEmbind = {
                 if (arguments.length !== argCount - 1) {
                     throwBindingError(humanName + ' called with ' + arguments.length + ' arguments, expected ' + (argCount-1));
                 }
-                var destructors = [];
-                var args = new Array(argCount);
-                args[0] = rawConstructor;
+                destructors.length = 0;
+                args.length = argCount;
                 for (var i = 1; i < argCount; ++i) {
                     args[i] = argTypes[i]['toWireType'](destructors, arguments[i - 1]);
                 }
@@ -2294,7 +2339,7 @@ var LibraryEmbind = {
     '$PureVirtualError', '$readLatin1String',
     '$registerInheritedInstance', '$requireHandle',
     '$requireRegisteredType', '$throwBindingError',
-    '$unregisterInheritedInstance'],
+    '$unregisterInheritedInstance', '$detachFinalizer', '$attachFinalizer'],
   _embind_create_inheriting_constructor: function(constructorName, wrapperType, properties) {
     constructorName = readLatin1String(constructorName);
     wrapperType = requireRegisteredType(wrapperType, 'wrapper');
@@ -2330,12 +2375,14 @@ var LibraryEmbind = {
         var inner = baseConstructor["implement"].apply(
             undefined,
             [this].concat(arraySlice.call(arguments)));
+        detachFinalizer(inner);
         var $$ = inner.$$;
         inner["notifyOnDestruction"]();
         $$.preservePointerOnDelete = true;
         Object.defineProperties(this, { $$: {
             value: $$
         }});
+        attachFinalizer(this);
         registerInheritedInstance(registeredClass, $$.ptr, this);
     };
 
@@ -2344,6 +2391,7 @@ var LibraryEmbind = {
             throwBindingError("Pass correct 'this' to __destruct");
         }
 
+        detachFinalizer(this);
         unregisterInheritedInstance(registeredClass, this.$$.ptr);
     };
 
