@@ -1,3 +1,8 @@
+// Copyright 2012 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
+
 /*global Module:true, Runtime*/
 /*global HEAP32*/
 /*global new_*/
@@ -95,6 +100,7 @@ var LibraryEmVal = {
     }
   },
 
+  _emval_decref__sig: 'vi',
   _emval_decref__deps: ['$emval_free_list', '$emval_handle_array'],
   _emval_decref: function(handle) {
     if (handle > 4 && 0 === --emval_handle_array[handle].refcount) {
@@ -146,7 +152,19 @@ var LibraryEmVal = {
         var obj = new constructor(arg0, arg1, arg2);
         return __emval_register(obj);
     } */
-
+#if DYNAMIC_EXECUTION == 0
+    var argsList = new Array(argCount + 1);
+    return function(constructor, argTypes, args) {
+      argsList[0] = constructor;
+      for (var i = 0; i < argCount; ++i) {
+        var argType = requireRegisteredType(HEAP32[(argTypes >> 2) + i], 'parameter ' + i);
+        argsList[i + 1] = argType.readValueFromPointer(args);
+        args += argType.argPackAdvance;
+      }
+      var obj = new (constructor.bind.apply(constructor, argsList));
+      return __emval_register(obj);
+    };
+#else
     var argsList = "";
     for(var i = 0; i < argCount; ++i) {
         argsList += (i!==0?", ":"")+"arg"+i; // 'arg0, arg1, ..., argn'
@@ -157,7 +175,7 @@ var LibraryEmVal = {
 
     for(var i = 0; i < argCount; ++i) {
         functionBody +=
-            "var argType"+i+" = requireRegisteredType(HEAP32[(argTypes >> 2) + "+i+"], \"parameter "+i+"\");\n" +
+            "var argType"+i+" = requireRegisteredType(Module['HEAP32'][(argTypes >>> 2) + "+i+"], \"parameter "+i+"\");\n" +
             "var arg"+i+" = argType"+i+".readValueFromPointer(args);\n" +
             "args += argType"+i+"['argPackAdvance'];\n";
     }
@@ -167,8 +185,9 @@ var LibraryEmVal = {
         "}\n";
 
     /*jshint evil:true*/
-    return (new Function("requireRegisteredType", "HEAP32", "__emval_register", functionBody))(
-        requireRegisteredType, HEAP32, __emval_register);
+    return (new Function("requireRegisteredType", "Module", "__emval_register", functionBody))(
+        requireRegisteredType, Module, __emval_register);
+#endif
   },
 
   _emval_new__deps: ['$craftEmvalAllocator', '$emval_newers', '$requireHandle'],
@@ -184,8 +203,43 @@ var LibraryEmVal = {
     return newer(handle, argTypes, args);
   },
 
+#if DYNAMIC_EXECUTION == 0
+  $emval_get_global: function() {
+    if (typeof globalThis === 'object') {
+      return globalThis;
+    }
+    function testGlobal(obj) {
+      obj['$$$embind_global$$$'] = obj;
+      var success = typeof $$$embind_global$$$ === 'object' && obj['$$$embind_global$$$'] === obj;
+      if (!success) {
+        delete obj['$$$embind_global$$$'];
+      }
+      return success;
+    }
+    if (typeof $$$embind_global$$$ === 'object') {
+      return $$$embind_global$$$;
+    }
+    if (typeof global === 'object' && testGlobal(global)) {
+      $$$embind_global$$$ = global;
+    } else if (typeof self === 'object' && testGlobal(self)) {
+      $$$embind_global$$$ = self; // This works for both "window" and "self" (Web Workers) global objects
+    }
+    if (typeof $$$embind_global$$$ === 'object') {
+      return $$$embind_global$$$;
+    }
+    throw Error('unable to get global object.');
+  },
+#else
   // appease jshint (technically this code uses eval)
-  $emval_get_global: function() { return (function(){return Function;})()('return this')(); },
+  $emval_get_global: function() {
+    if (typeof globalThis === 'object') {
+      return globalThis;
+    }
+    return (function(){
+      return Function;
+    })()('return this')();
+  },
+#endif
   _emval_get_global__deps: ['_emval_register', '$getStringOrSymbol', '$emval_get_global'],
   _emval_get_global: function(name) {
     if(name===0){
@@ -227,20 +281,41 @@ var LibraryEmVal = {
     return returnType['toWireType'](destructors, handle);
   },
 
-  _emval_equals__deps: ['_emval_register', '$requireHandle'],
-  _emval_equals: function(first, second ){
+  _emval_equals__deps: ['$requireHandle'],
+  _emval_equals: function(first, second) {
     first = requireHandle(first);
     second = requireHandle(second);
-    return first==second;
+    return first == second;
   },
 
-  _emval_strictly_equals__deps: ['_emval_register', '$requireHandle'],
-  _emval_strictly_equals: function(first, second ){
+  _emval_strictly_equals__deps: ['$requireHandle'],
+  _emval_strictly_equals: function(first, second) {
     first = requireHandle(first);
     second = requireHandle(second);
-    return first===second;
+    return first === second;
   },
 
+  _emval_greater_than__deps: ['$requireHandle'],
+  _emval_greater_than: function(first, second) {
+    first = requireHandle(first);
+    second = requireHandle(second);
+    return first > second;
+  },
+
+  _emval_less_than__deps: ['$requireHandle'],
+  _emval_less_than: function(first, second) {
+    first = requireHandle(first);
+    second = requireHandle(second);
+    return first < second;
+  },
+
+  _emval_not__deps: ['$requireHandle'],
+  _emval_not: function(object) {
+    object = requireHandle(object);
+    return !object;
+  },
+
+  _emval_call__sig: 'iiiii',
   _emval_call__deps: ['_emval_lookupTypes', '_emval_register', '$requireHandle'],
   _emval_call: function(handle, argCount, argTypes, argv) {
     handle = requireHandle(handle);
@@ -258,7 +333,7 @@ var LibraryEmVal = {
   },
 
   _emval_lookupTypes__deps: ['$requireRegisteredType'],
-  _emval_lookupTypes: function(argCount, argTypes, argWireTypes) {
+  _emval_lookupTypes: function(argCount, argTypes) {
     var a = new Array(argCount);
     for (var i = 0; i < argCount; ++i) {
         a[i] = requireRegisteredType(
@@ -291,6 +366,25 @@ var LibraryEmVal = {
     var types = __emval_lookupTypes(argCount, argTypes);
 
     var retType = types[0];
+#if DYNAMIC_EXECUTION == 0
+    var argN = new Array(argCount - 1);
+    var invokerFunction = function(handle, name, destructors, args) {
+      var offset = 0;
+      for (var i = 0; i < argCount - 1; ++i) {
+        argN[i] = types[i + 1].readValueFromPointer(args + offset);
+        offset += types[i + 1].argPackAdvance;
+      }
+      var rv = handle[name].apply(handle, argN);
+      for (var i = 0; i < argCount - 1; ++i) {
+        if (types[i + 1].deleteObject) {
+          types[i + 1].deleteObject(argN[i]);
+        }
+      }
+      if (!retType.isVoid) {
+        return retType.toWireType(destructors, rv);
+      }
+    };
+#else
     var signatureName = retType.name + "_$" + types.slice(1).map(function (t) { return t.name; }).join("_") + "$";
 
     var params = ["retType"];
@@ -325,11 +419,12 @@ var LibraryEmVal = {
         functionBody +=
         "    return retType.toWireType(destructors, rv);\n";
     }
-    functionBody += 
+    functionBody +=
         "};\n";
 
     params.push(functionBody);
     var invokerFunction = new_(Function, params).apply(null, args);
+#endif
     return __emval_addMethodCaller(invokerFunction);
   },
 
@@ -354,6 +449,55 @@ var LibraryEmVal = {
     handle = requireHandle(handle);
     return __emval_register(typeof handle);
   },
+
+  _emval_instanceof__deps: ['$requireHandle'],
+  _emval_instanceof: function(object, constructor) {
+    object = requireHandle(object);
+    constructor = requireHandle(constructor);
+    return object instanceof constructor;
+  },
+
+  _emval_is_number__deps: ['$requireHandle'],
+  _emval_is_number: function(handle) {
+    handle = requireHandle(handle);
+    return typeof handle === 'number';
+  },
+
+  _emval_is_string__deps: ['$requireHandle'],
+  _emval_is_string: function(handle) {
+    handle = requireHandle(handle);
+    return typeof handle === 'string';
+  },
+
+  _emval_in__deps: ['$requireHandle'],
+  _emval_in: function(item, object) {
+    item = requireHandle(item);
+    object = requireHandle(object);
+    return item in object;
+  },
+
+  _emval_delete__deps: ['$requireHandle'],
+  _emval_delete: function(object, property) {
+    object = requireHandle(object);
+    property = requireHandle(property);
+    return delete object[property];
+  },
+
+  _emval_throw__deps: ['$requireHandle'],
+  _emval_throw: function(object) {
+    object = requireHandle(object);
+    throw object;
+  },
+
+#if ASYNCIFY
+  _emval_await__deps: ['$requireHandle', '_emval_register', '$Asyncify'],
+  _emval_await: function(promise) {
+    return Asyncify.handleAsync(function () {
+      promise = requireHandle(promise);
+      return promise.then(__emval_register);
+    });
+  },
+#endif
 };
 
 mergeInto(LibraryManager.library, LibraryEmVal);
