@@ -46,7 +46,7 @@ var LibraryDylink = {
 
   $GOT: {},
 
-  // Greate globals to each imported symbol.  These are all initialized to zero
+  // Create globals to each imported symbol.  These are all initialized to zero
   // and get assigned later in `updateGOT`
   $GOTHandler__deps: ['$GOT'],
   $GOTHandler: {
@@ -61,13 +61,28 @@ var LibraryDylink = {
     }
   },
 
-  $updateGOT__deps: ['$GOT'],
+  $isInternalSym: function(symName) {
+    // TODO: find a way to mark these in the binary or avoid exporting them.
+    return [
+      '__cpp_exception',
+      '__wasm_apply_data_relocs',
+      '__dso_handle',
+      '__set_stack_limits'
+    ].indexOf(symName) != -1
+#if SPLIT_MODULE
+        // Exports synthesized by wasm-split should be prefixed with '%'
+        || symName[0] == '%'
+#endif
+    ;
+  },
+
+  $updateGOT__deps: ['$GOT', '$isInternalSym'],
   $updateGOT: function(exports) {
 #if DYLINK_DEBUG
     err("updateGOT: " + Object.keys(exports).length);
 #endif
     for (var symName in exports) {
-      if (symName == '__cpp_exception' || symName == '__dso_handle' || symName == '__wasm_apply_relocs') {
+      if (isInternalSym(symName)) {
         continue;
       }
 
@@ -116,6 +131,13 @@ var LibraryDylink = {
 
     for (var e in exports) {
       var value = exports[e];
+#if SPLIT_MODULE
+      // Do not modify exports synthesized by wasm-split
+      if (e.startsWith('%')) {
+        relocated[e] = value
+        continue;
+      }
+#endif
       if (typeof value === 'object') {
         // a breaking change in the wasm spec, globals are now objects
         // https://github.com/WebAssembly/mutable-global/issues/1
@@ -195,7 +217,7 @@ var LibraryDylink = {
     loadedLibNames: {},
   },
 
-  // Dynmamic version of shared.py:make_invoke.  This is needed for invokes
+  // Dynamic version of shared.py:make_invoke.  This is needed for invokes
   // that originate from side modules since these are not known at JS
   // generation time.
   $createInvokeFunction: function(sig) {
@@ -236,7 +258,7 @@ var LibraryDylink = {
     return ret;
   },
 
-  // fetchBinary fetches binaray data @ url. (async)
+  // fetchBinary fetches binary data @ url. (async)
   $fetchBinary: function(url) {
     return fetch(url, { credentials: 'same-origin' }).then(function(response) {
       if (!response['ok']) {
@@ -248,15 +270,10 @@ var LibraryDylink = {
     });
   },
 
-  // Loads a side module from binary data. Returns the module's exports or a
-  // progise that resolves to its exports if the loadAsync flag is set.
-  $loadSideModule__deps: ['$loadDynamicLibrary', '$createInvokeFunction', '$getMemory', '$relocateExports', '$resolveGlobalSymbol', '$GOTHandler'],
-  $loadSideModule: function(binary, flags) {
-    var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
-    assert(int32View[0] == 0x6d736100, 'need to see wasm magic number'); // \0asm
-    // we should see the dylink section right after the magic number and wasm version
-    assert(binary[8] === 0, 'need the dylink section to be first')
-    var next = 9;
+  // returns the side module metadata as an object
+  // { memorySize, memoryAlign, tableSize, tableAlign, neededDynlibs}
+  $getDylinkMetadata: function(binary) {
+    var next = 0;
     function getLEB() {
       var ret = 0;
       var mul = 1;
@@ -268,32 +285,93 @@ var LibraryDylink = {
       }
       return ret;
     }
-    var sectionSize = getLEB();
-    assert(binary[next] === 6);                 next++; // size of "dylink" string
-    assert(binary[next] === 'd'.charCodeAt(0)); next++;
-    assert(binary[next] === 'y'.charCodeAt(0)); next++;
-    assert(binary[next] === 'l'.charCodeAt(0)); next++;
-    assert(binary[next] === 'i'.charCodeAt(0)); next++;
-    assert(binary[next] === 'n'.charCodeAt(0)); next++;
-    assert(binary[next] === 'k'.charCodeAt(0)); next++;
-    var memorySize = getLEB();
-    var memoryAlign = getLEB();
-    var tableSize = getLEB();
-    var tableAlign = getLEB();
 
-    // shared libraries this module needs. We need to load them first, so that
-    // current module could resolve its imports. (see tools/shared.py
-    // WebAssembly.make_shared_library() for "dylink" section extension format)
-    var neededDynlibsCount = getLEB();
-    var neededDynlibs = [];
-    for (var i = 0; i < neededDynlibsCount; ++i) {
-      var nameLen = getLEB();
-      var nameUTF8 = binary.subarray(next, next + nameLen);
-      next += nameLen;
-      var name = UTF8ArrayToString(nameUTF8, 0);
-      neededDynlibs.push(name);
+    function parseDylinkSection() {
+      var customSection = {};
+      customSection.memorySize = getLEB();
+      customSection.memoryAlign = getLEB();
+      customSection.tableSize = getLEB();
+      customSection.tableAlign = getLEB();
+      // shared libraries this module needs. We need to load them first, so that
+      // current module could resolve its imports. (see tools/shared.py
+      // WebAssembly.make_shared_library() for "dylink" section extension format)
+      var neededDynlibsCount = getLEB();
+      customSection.neededDynlibs = [];
+      for (var i = 0; i < neededDynlibsCount; ++i) {
+        var nameLen = getLEB();
+        var nameUTF8 = binary.subarray(next, next + nameLen);
+        next += nameLen;
+        var name = UTF8ArrayToString(nameUTF8, 0);
+        customSection.neededDynlibs.push(name);
+      }
+      return customSection;
     }
 
+    if (binary instanceof WebAssembly.Module) {
+      var dylinkSection = WebAssembly.Module.customSections(binary, "dylink");
+      assert(dylinkSection.length != 0, 'need dylink section');
+      binary = new Int8Array(dylinkSection[0]);
+    } else {
+      var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
+      assert(int32View[0] == 0x6d736100, 'need to see wasm magic number'); // \0asm
+      // we should see the dylink section right after the magic number and wasm version
+      assert(binary[8] === 0, 'need the dylink section to be first')
+      next = 9;
+      getLEB(); //section size
+      assert(binary[next] === 6);                 next++; // size of "dylink" string
+      assert(binary[next] === 'd'.charCodeAt(0)); next++;
+      assert(binary[next] === 'y'.charCodeAt(0)); next++;
+      assert(binary[next] === 'l'.charCodeAt(0)); next++;
+      assert(binary[next] === 'i'.charCodeAt(0)); next++;
+      assert(binary[next] === 'n'.charCodeAt(0)); next++;
+      assert(binary[next] === 'k'.charCodeAt(0)); next++;
+    }
+
+    return parseDylinkSection();
+  },
+
+  // Module.symbols <- libModule.symbols (flags.global handler)
+  $mergeLibSymbols__deps: ['$asmjsMangle'],
+  $mergeLibSymbols: function(libModule, libName) {
+    // add symbols into global namespace TODO: weak linking etc.
+    for (var sym in libModule) {
+      if (!libModule.hasOwnProperty(sym)) {
+        continue;
+      }
+
+      // When RTLD_GLOBAL is enable, the symbols defined by this shared object will be made
+      // available for symbol resolution of subsequently loaded shared objects.
+      //
+      // We should copy the symbols (which include methods and variables) from SIDE_MODULE to MAIN_MODULE.
+
+      var module_sym = asmjsMangle(sym);
+
+      if (!Module.hasOwnProperty(module_sym)) {
+        Module[module_sym] = libModule[sym];
+      }
+#if ASSERTIONS == 2
+      else {
+        var curr = Module[sym], next = libModule[sym];
+        // don't warn on functions - might be odr, linkonce_odr, etc.
+        if (!(typeof curr === 'function' && typeof next === 'function')) {
+          err("warning: symbol '" + sym + "' from '" + libName + "' already exists (duplicate symbol? or weak linking, which isn't supported yet?)"); // + [curr, ' vs ', next]);
+        }
+      }
+#endif
+    }
+  },
+
+  // Loads a side module from binary data or compiled Module. Returns the module's exports or a
+  // promise that resolves to its exports if the loadAsync flag is set.
+  $loadWebAssemblyModule__deps: ['$loadDynamicLibrary', '$createInvokeFunction', '$getMemory', '$relocateExports', '$resolveGlobalSymbol', '$GOTHandler', '$getDylinkMetadata'],
+  $loadWebAssemblyModule: function(binary, flags) {
+    var metadata = getDylinkMetadata(binary);
+    var memorySize = metadata.memorySize;
+    var memoryAlign = metadata.memoryAlign;
+    var tableSize = metadata.tableSize;
+    var tableAlign = metadata.tableAlign;
+    var neededDynlibs = metadata.neededDynlibs;
+
     // loadModule loads the wasm module after all its dependencies have been loaded.
     // can be called both sync/async.
     function loadModule() {
@@ -395,31 +473,32 @@ var LibraryDylink = {
         'env': proxy,
         {{{ WASI_MODULE_NAME }}}: proxy,
       };
-#if ASSERTIONS
-      var oldTable = [];
-      for (var i = 0; i < tableBase; i++) {
-        oldTable.push(table.get(i));
-      }
-#endif
 
       function postInstantiation(instance) {
 #if ASSERTIONS
         // the table should be unchanged
         assert(table === originalTable);
         assert(table === wasmTable);
-        // the old part of the table should be unchanged
-        for (var i = 0; i < tableBase; i++) {
-          assert(table.get(i) === oldTable[i], 'old table entries must remain the same');
-        }
-        // verify that the new table region was filled in
-        for (var i = 0; i < tableSize; i++) {
-          assert(table.get(tableBase + i) !== undefined, 'table entry was not filled in');
-        }
 #endif
+        // add new entries to functionsInTableMap
+        for (var i = 0; i < tableSize; i++) {
+          var item = table.get(tableBase + i);
+#if ASSERTIONS
+          // verify that the new table region was filled in
+          assert(item !== undefined, 'table entry was not filled in');
+#endif
+          // Ignore null values.
+          if (item) {
+            functionsInTableMap.set(item, tableBase + i);
+          }
+        }
         moduleExports = relocateExports(instance.exports, memoryBase);
         if (!flags.allowUndefined) {
           reportUndefinedSymbols();
         }
+#if STACK_OVERFLOW_CHECK >= 2
+        moduleExports['__set_stack_limits']({{{ STACK_BASE }}} , {{{ STACK_MAX }}});
+#endif
         // initialize the module
         var init = moduleExports['__post_instantiate'];
         if (init) {
@@ -434,20 +513,27 @@ var LibraryDylink = {
       }
 
       if (flags.loadAsync) {
+        if (binary instanceof WebAssembly.Module) {
+          var instance = new WebAssembly.Instance(binary, info);
+          return Promise.resolve(postInstantiation(instance));
+        }
         return WebAssembly.instantiate(binary, info).then(function(result) {
           return postInstantiation(result.instance);
         });
       }
 
-      var instance = new WebAssembly.Instance(new WebAssembly.Module(binary), info);
+      var module = binary instanceof WebAssembly.Module ? binary : new WebAssembly.Module(binary);
+      var instance = new WebAssembly.Instance(module, info);
       return postInstantiation(instance);
     }
 
     // now load needed libraries and the module itself.
     if (flags.loadAsync) {
-      return Promise.all(neededDynlibs.map(function(dynNeeded) {
-        return loadDynamicLibrary(dynNeeded, flags);
-      })).then(function() {
+      return neededDynlibs.reduce(function(chain, dynNeeded) {
+        return chain.then(function() {
+          return loadDynamicLibrary(dynNeeded, flags);
+        });
+      }, Promise.resolve()).then(function() {
         return loadModule();
       });
     }
@@ -478,7 +564,7 @@ var LibraryDylink = {
   // If a library was already loaded, it is not loaded a second time. However
   // flags.global and flags.nodelete are handled every time a load request is made.
   // Once a library becomes "global" or "nodelete", it cannot be removed or unloaded.
-  $loadDynamicLibrary__deps: ['$LDSO', '$loadSideModule', '$asmjsMangle', '$fetchBinary'],
+  $loadDynamicLibrary__deps: ['$LDSO', '$loadWebAssemblyModule', '$asmjsMangle', '$fetchBinary', '$isInternalSym', '$mergeLibSymbols'],
   $loadDynamicLibrary: function(lib, flags) {
     if (lib == '__main__' && !LDSO.loadedLibNames[lib]) {
       LDSO.loadedLibs[-1] = {
@@ -505,7 +591,7 @@ var LibraryDylink = {
         dso.global = true;
         if (dso.module !== 'loading') {
           // ^^^ if module is 'loading' - symbols merging will be eventually done by the loader.
-          mergeLibSymbols(dso.module)
+          mergeLibSymbols(dso.module, lib)
         }
       }
       // same for "nodelete"
@@ -533,7 +619,7 @@ var LibraryDylink = {
       if (flags.fs) {
         var libData = flags.fs.readFile(libFile, {encoding: 'binary'});
         if (!(libData instanceof Uint8Array)) {
-          libData = new Uint8Array(lib_data);
+          libData = new Uint8Array(libData);
         }
         return flags.loadAsync ? Promise.resolve(libData) : libData;
       }
@@ -557,48 +643,17 @@ var LibraryDylink = {
       // module not preloaded - load lib data and create new module from it
       if (flags.loadAsync) {
         return loadLibData(lib).then(function(libData) {
-          return loadSideModule(libData, flags);
+          return loadWebAssemblyModule(libData, flags);
         });
       }
 
-      return loadSideModule(loadLibData(lib), flags);
-    }
-
-    // Module.symbols <- libModule.symbols (flags.global handler)
-    function mergeLibSymbols(libModule) {
-      // add symbols into global namespace TODO: weak linking etc.
-      for (var sym in libModule) {
-        if (!libModule.hasOwnProperty(sym)) {
-          continue;
-        }
-
-        // When RTLD_GLOBAL is enable, the symbols defined by this shared object will be made
-        // available for symbol resolution of subsequently loaded shared objects.
-        //
-        // We should copy the symbols (which include methods and variables) from
-        // SIDE_MODULE to MAIN_MODULE.
-
-        var module_sym = asmjsMangle(sym);
-
-        if (!Module.hasOwnProperty(module_sym)) {
-          Module[module_sym] = libModule[sym];
-        }
-#if ASSERTIONS == 2
-        else {
-          var curr = Module[sym], next = libModule[sym];
-          // don't warn on functions - might be odr, linkonce_odr, etc.
-          if (!(typeof curr === 'function' && typeof next === 'function')) {
-            err("warning: symbol '" + sym + "' from '" + lib + "' already exists (duplicate symbol? or weak linking, which isn't supported yet?)");
-          }
-        }
-#endif
-      }
+      return loadWebAssemblyModule(loadLibData(lib), flags);
     }
 
     // module for lib is loaded - update the dso & global namespace
     function moduleLoaded(libModule) {
       if (dso.global) {
-        mergeLibSymbols(libModule);
+        mergeLibSymbols(libModule, lib);
       }
       dso.module = libModule;
     }
@@ -635,9 +690,11 @@ var LibraryDylink = {
     if (!readBinary) {
       // we can't read binary data synchronously, so preload
       addRunDependency('preloadDylibs');
-      Promise.all(libs.map(function(lib) {
-        return loadDynamicLibrary(lib, {loadAsync: true, global: true, nodelete: true, allowUndefined: true});
-      })).then(function() {
+      libs.reduce(function(chain, lib) {
+        return chain.then(function() {
+          return loadDynamicLibrary(lib, {loadAsync: true, global: true, nodelete: true, allowUndefined: true});
+        });
+      }, Promise.resolve()).then(function() {
         // we got them all, wonderful
         removeRunDependency('preloadDylibs');
         reportUndefinedSymbols();
