@@ -1,7 +1,8 @@
-// Copyright 2015 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2015 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 #if LZ4
 mergeInto(LibraryManager.library, {
@@ -14,21 +15,21 @@ mergeInto(LibraryManager.library, {
     init: function() {
       if (LZ4.codec) return;
       LZ4.codec = (function() {
-        {{{ read('mini-lz4.js') }}};
+        {{{ read('../third_party/mini-lz4.js') }}};
         return MiniLZ4;
       })();
       LZ4.CHUNK_SIZE = LZ4.codec.CHUNK_SIZE;
     },
-    loadPackage: function (pack) {
+    loadPackage: function (pack, preloadPlugin) {
       LZ4.init();
       var compressedData = pack['compressedData'];
       if (!compressedData) compressedData = LZ4.codec.compressPackage(pack['data']);
-      assert(compressedData.cachedIndexes.length === compressedData.cachedChunks.length);
-      for (var i = 0; i < compressedData.cachedIndexes.length; i++) {
-        compressedData.cachedIndexes[i] = -1;
-        compressedData.cachedChunks[i] = compressedData.data.subarray(compressedData.cachedOffset + i*LZ4.CHUNK_SIZE,
-                                                                      compressedData.cachedOffset + (i+1)*LZ4.CHUNK_SIZE);
-        assert(compressedData.cachedChunks[i].length === LZ4.CHUNK_SIZE);
+      assert(compressedData['cachedIndexes'].length === compressedData['cachedChunks'].length);
+      for (var i = 0; i < compressedData['cachedIndexes'].length; i++) {
+        compressedData['cachedIndexes'][i] = -1;
+        compressedData['cachedChunks'][i] = compressedData['data'].subarray(compressedData['cachedOffset'] + i*LZ4.CHUNK_SIZE,
+                                                                      compressedData['cachedOffset'] + (i+1)*LZ4.CHUNK_SIZE);
+        assert(compressedData['cachedChunks'][i].length === LZ4.CHUNK_SIZE);
       }
       pack['metadata'].files.forEach(function(file) {
         var dir = PATH.dirname(file.filename);
@@ -41,6 +42,31 @@ mergeInto(LibraryManager.library, {
           end: file.end,
         });
       });
+      // Preload files if necessary. This code is largely similar to
+      // createPreloadedFile in library_fs.js. However, a main difference here
+      // is that we only decompress the file if it can be preloaded.
+      // Abstracting out the common parts seems to be more effort than it is
+      // worth.
+      if (preloadPlugin) {
+        Browser.init();
+        pack['metadata'].files.forEach(function(file) {
+          var handled = false;
+          var fullname = file.filename;
+          Module['preloadPlugins'].forEach(function(plugin) {
+            if (handled) return;
+            if (plugin['canHandle'](fullname)) {
+              var dep = getUniqueRunDependency('fp ' + fullname);
+              addRunDependency(dep);
+              var finish = function() {
+                removeRunDependency(dep);
+              }
+              var byteArray = FS.readFile(fullname);
+              plugin['handle'](byteArray, fullname, finish, finish);
+              handled = true;
+            }
+          });
+        });
+      }
     },
     createNode: function (parent, name, mode, dev, contents, mtime) {
       var node = FS.createNode(parent, name, mode);
@@ -65,7 +91,7 @@ mergeInto(LibraryManager.library, {
       getattr: function(node) {
         return {
           dev: 1,
-          ino: undefined,
+          ino: node.id,
           mode: node.mode,
           nlink: 1,
           uid: 0,
@@ -125,32 +151,32 @@ mergeInto(LibraryManager.library, {
           var desired = length - written;
           //console.log('current read: ' + ['start', start, 'desired', desired]);
           var chunkIndex = Math.floor(start / LZ4.CHUNK_SIZE);
-          var compressedStart = compressedData.offsets[chunkIndex];
-          var compressedSize = compressedData.sizes[chunkIndex];
+          var compressedStart = compressedData['offsets'][chunkIndex];
+          var compressedSize = compressedData['sizes'][chunkIndex];
           var currChunk;
-          if (compressedData.successes[chunkIndex]) {
-            var found = compressedData.cachedIndexes.indexOf(chunkIndex);
+          if (compressedData['successes'][chunkIndex]) {
+            var found = compressedData['cachedIndexes'].indexOf(chunkIndex);
             if (found >= 0) {
-              currChunk = compressedData.cachedChunks[found];
+              currChunk = compressedData['cachedChunks'][found];
             } else {
               // decompress the chunk
-              compressedData.cachedIndexes.pop();
-              compressedData.cachedIndexes.unshift(chunkIndex);
-              currChunk = compressedData.cachedChunks.pop();
-              compressedData.cachedChunks.unshift(currChunk);
-              if (compressedData.debug) {
+              compressedData['cachedIndexes'].pop();
+              compressedData['cachedIndexes'].unshift(chunkIndex);
+              currChunk = compressedData['cachedChunks'].pop();
+              compressedData['cachedChunks'].unshift(currChunk);
+              if (compressedData['debug']) {
                 console.log('decompressing chunk ' + chunkIndex);
                 Module['decompressedChunks'] = (Module['decompressedChunks'] || 0) + 1;
               }
-              var compressed = compressedData.data.subarray(compressedStart, compressedStart + compressedSize);
+              var compressed = compressedData['data'].subarray(compressedStart, compressedStart + compressedSize);
               //var t = Date.now();
               var originalSize = LZ4.codec.uncompress(compressed, currChunk);
               //console.log('decompress time: ' + (Date.now() - t));
-              if (chunkIndex < compressedData.successes.length-1) assert(originalSize === LZ4.CHUNK_SIZE); // all but the last chunk must be full-size
+              if (chunkIndex < compressedData['successes'].length-1) assert(originalSize === LZ4.CHUNK_SIZE); // all but the last chunk must be full-size
             }
           } else {
             // uncompressed
-            currChunk = compressedData.data.subarray(compressedStart, compressedStart + LZ4.CHUNK_SIZE);
+            currChunk = compressedData['data'].subarray(compressedStart, compressedStart + LZ4.CHUNK_SIZE);
           }
           var startInChunk = start % LZ4.CHUNK_SIZE;
           var endInChunk = Math.min(startInChunk + desired, LZ4.CHUNK_SIZE);
@@ -165,9 +191,9 @@ mergeInto(LibraryManager.library, {
       },
       llseek: function (stream, offset, whence) {
         var position = offset;
-        if (whence === 1) {  // SEEK_CUR.
+        if (whence === {{{ cDefine('SEEK_CUR') }}}) {
           position += stream.position;
-        } else if (whence === 2) {  // SEEK_END.
+        } else if (whence === {{{ cDefine('SEEK_END') }}}) {
           if (FS.isFile(stream.node.mode)) {
             position += stream.node.size;
           }
