@@ -85,6 +85,11 @@
         ShaderModuleSPIRVDescriptor: 5,
         ShaderModuleWGSLDescriptor: 6,
     },
+    MapMode: {
+        None: 0,
+        Read: 1,
+        Write: 2
+    }
   };
   return null;
 })(); }}}
@@ -162,10 +167,10 @@ var LibraryWebGPU = {
 
     makeColor: function(ptr) {
       return {
-        "r": {{{ makeGetValue('ptr', 0, 'float') }}},
-        "g": {{{ makeGetValue('ptr', 4, 'float') }}},
-        "b": {{{ makeGetValue('ptr', 8, 'float') }}},
-        "a": {{{ makeGetValue('ptr', 12, 'float') }}},
+        "r": {{{ makeGetValue('ptr', 0, 'double') }}},
+        "g": {{{ makeGetValue('ptr', 8, 'double') }}},
+        "b": {{{ makeGetValue('ptr', 16, 'double') }}},
+        "a": {{{ makeGetValue('ptr', 24, 'double') }}},
       };
     },
 
@@ -192,6 +197,7 @@ var LibraryWebGPU = {
           {{{ makeGetValue('ptr', C_STRUCTS.WGPUTextureCopyView.texture, '*') }}}),
         "mipLevel": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUTextureCopyView.mipLevel, '*') }}},
         "origin": WebGPU.makeOrigin3D(ptr + {{{ C_STRUCTS.WGPUTextureCopyView.origin }}}),
+        "aspect": WebGPU.TextureAspect[{{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUTextureCopyView.aspect) }}}],
       };
     },
 
@@ -237,12 +243,14 @@ var LibraryWebGPU = {
       'clamp-to-edge',
     ],
     BindingType: [
+      undefined,
       'uniform-buffer',
       'storage-buffer',
       'readonly-storage-buffer',
       'sampler',
       'comparison-sampler',
       'sampled-texture',
+      'multisampled-texture',
       'readonly-storage-texture',
       'writeonly-storage-texture',
     ],
@@ -268,11 +276,19 @@ var LibraryWebGPU = {
       'min',
       'max',
     ],
+    BufferBindingType: [
+      undefined,
+      'uniform',
+      'storage',
+      'read-only-storage',
+    ],
     BufferMapAsyncStatus: [
       'success',
       'error',
       'unknown',
       'device-lost',
+      'destroyed-before-callback',
+      'unmapped-before-callback'
     ],
     CompareFunction: [
       undefined,
@@ -317,6 +333,7 @@ var LibraryWebGPU = {
       'cw',
     ],
     IndexFormat: [
+      undefined,
       'uint16',
       'uint32',
     ],
@@ -370,6 +387,7 @@ var LibraryWebGPU = {
       'float',
       'sint',
       'uint',
+      'depth-comparison',
     ],
     TextureDimension: [
       '1d',
@@ -403,7 +421,8 @@ var LibraryWebGPU = {
       'bgra8unorm',
       'bgra8unorm-srgb',
       'rgb10a2unorm',
-      'rg11b10float',
+      'rg11b10ufloat',
+      'rgb9e5ufloat',
       'rg32float',
       'rg32uint',
       'rg32sint',
@@ -427,10 +446,18 @@ var LibraryWebGPU = {
       'bc5-rg-unorm',
       'bc5-rg-snorm',
       'bc6h-rgb-ufloat',
-      'bc6h-rgb-sfloat',
+      'bc6h-rgb-float',
       'bc7-rgba-unorm',
       'bc7-rgba-unorm-srgb',
     ],
+    TextureSampleType: [
+      undefined,
+      'float',
+      'unfilterable-float',
+      'depth',
+      'sint',
+      'uint',
+    ],
     TextureViewDimension: [
       undefined,
       '1d',
@@ -440,6 +467,17 @@ var LibraryWebGPU = {
       'cube-array',
       '3d',
     ],
+    SamplerBindingType: [
+      undefined,
+      'filtering',
+      'non-filtering',
+      'comparison',
+    ],
+    StorageTextureAccess: [
+      undefined,
+      'read-only',
+      'write-only',
+    ],
     VertexFormat: [
       'uchar2',
       'uchar4',
@@ -561,7 +599,7 @@ var LibraryWebGPU = {
     var id = WebGPU.mgrBuffer.create(device["createBuffer"](desc));
     if (mappedAtCreation) {
       var bufferWrapper = WebGPU.mgrBuffer.objects[id];
-      bufferWrapper.mapMode = 2 /* WGPUMapMode_Write */;
+      bufferWrapper.mapMode = {{{ gpu.MapMode.Write }}};
       bufferWrapper.onUnmap = [];
     }
     return id;
@@ -620,9 +658,67 @@ var LibraryWebGPU = {
   wgpuDeviceCreateBindGroupLayout: function(deviceId, descriptor) {
     {{{ gpu.makeCheckDescriptor('descriptor') }}}
 
-    function makeEntry(entryPtr) {
+    function makeBufferEntry(entryPtr) {
       {{{ gpu.makeCheck('entryPtr') }}}
 
+      var typeInt =
+        {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBufferBindingLayout.type) }}};
+      if (typeInt === 0) return undefined;
+
+      return {
+        "type": WebGPU.BufferBindingType[typeInt],
+        "hasDynamicOffset":
+          {{{ gpu.makeGetBool('entryPtr', C_STRUCTS.WGPUBufferBindingLayout.hasDynamicOffset) }}},
+        "minBindingSize":
+          {{{ gpu.makeGetU64('entryPtr', C_STRUCTS.WGPUBufferBindingLayout.minBindingSize) }}},
+      };
+    }
+
+    function makeSamplerEntry(entryPtr) {
+      {{{ gpu.makeCheck('entryPtr') }}}
+
+      var typeInt =
+        {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUSamplerBindingLayout.type) }}};
+      if (typeInt === 0) return undefined;
+
+      return {
+        "type": WebGPU.SamplerBindingType[typeInt],
+      };
+    }
+
+    function makeTextureEntry(entryPtr) {
+      {{{ gpu.makeCheck('entryPtr') }}}
+
+      var sampleTypeInt =
+        {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUTextureBindingLayout.sampleType) }}};
+      if (sampleTypeInt === 0) return undefined;
+
+      return {
+        "sampleType": WebGPU.TextureSampleType[sampleTypeInt],
+        "viewDimension": WebGPU.TextureViewDimension[
+          {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUTextureBindingLayout.viewDimension) }}}],
+        "multisampled":
+          {{{ gpu.makeGetBool('entryPtr', C_STRUCTS.WGPUTextureBindingLayout.multisampled) }}},
+      };
+    }
+
+    function makeStorageTextureEntry(entryPtr) {
+      {{{ gpu.makeCheck('entryPtr') }}}
+
+      var accessInt =
+        {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUStorageTextureBindingLayout.access) }}}
+      if (accessInt === 0) return undefined;
+
+      return {
+        "access": WebGPU.StorageTextureAccess[accessInt],
+        "format": WebGPU.TextureFormat[
+          {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUStorageTextureBindingLayout.format) }}}],
+        "viewDimension": WebGPU.TextureViewDimension[
+          {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUStorageTextureBindingLayout.viewDimension) }}}],
+      };
+    }
+
+    function makeDeprecatedEntry(entryPtr) {
       return {
         "binding":
           {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.binding) }}},
@@ -636,8 +732,6 @@ var LibraryWebGPU = {
           {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.textureComponentType) }}}],
         "storageTextureFormat": WebGPU.TextureFormat[
           {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.storageTextureFormat) }}}],
-        "multisampled":
-          {{{ gpu.makeGetBool('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.multisampled) }}},
         "hasDynamicOffset":
           {{{ gpu.makeGetBool('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.hasDynamicOffset) }}},
         "minBufferBindingSize":
@@ -645,6 +739,27 @@ var LibraryWebGPU = {
       };
     }
 
+    function makeEntry(entryPtr) {
+      {{{ gpu.makeCheck('entryPtr') }}}
+
+      var typeInt =
+        {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.type) }}};
+
+      if (typeInt !== 0)
+        return makeDeprecatedEntry(entryPtr);
+
+      return {
+        "binding":
+          {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.binding) }}},
+        "visibility":
+          {{{ gpu.makeGetU32('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.visibility) }}},
+        "buffer": makeBufferEntry(entryPtr + {{{ C_STRUCTS.WGPUBindGroupLayoutEntry.buffer }}}),
+        "sampler": makeSamplerEntry(entryPtr + {{{ C_STRUCTS.WGPUBindGroupLayoutEntry.sampler }}}),
+        "texture": makeTextureEntry(entryPtr + {{{ C_STRUCTS.WGPUBindGroupLayoutEntry.texture }}}),
+        "storageTexture": makeStorageTextureEntry(entryPtr + {{{ C_STRUCTS.WGPUBindGroupLayoutEntry.storageTexture }}}),
+      };
+    }
+
     function makeEntries(count, entriesPtrs) {
       var entries = [];
       for (var i = 0; i < count; ++i) {
@@ -1292,14 +1407,14 @@ var LibraryWebGPU = {
 
   // wgpuComputePipeline
 
-  wgpuComputePipelineGetBindGroupLayout(pipelineId, groupIndex) {
+  wgpuComputePipelineGetBindGroupLayout: function(pipelineId, groupIndex) {
     var pipeline = WebGPU.mgrComputePipeline.get(pipelineId);
     return WebGPU.mgrBindGroupLayout.create(pipeline["getBindGroupLayout"](groupIndex));
   },
 
   // wgpuRenderPipeline
 
-  wgpuRenderPipelineGetBindGroupLayout(pipelineId, groupIndex) {
+  wgpuRenderPipelineGetBindGroupLayout: function(pipelineId, groupIndex) {
     var pipeline = WebGPU.mgrRenderPipeline.get(pipelineId);
     return WebGPU.mgrBindGroupLayout.create(pipeline["getBindGroupLayout"](groupIndex));
   },
@@ -1337,7 +1452,7 @@ var LibraryWebGPU = {
     // TODO: if the sentinel value becomes WGPU_WHOLE_SIZE instead of 0, update this.
     if (size === 0) size = undefined;
 
-    if (bufferWrapper.mapMode !== 2 /* WGPUMapMode_Write */) {
+    if (bufferWrapper.mapMode !== {{{ gpu.MapMode.Write }}}) {
 #if ASSERTIONS
       abort("GetMappedRange called, but buffer not mapped for writing");
 #endif
@@ -1510,10 +1625,19 @@ var LibraryWebGPU = {
     var color = WebGPU.makeColor(colorPtr);
     pass["setBlendColor"](color);
   },
-  wgpuRenderPassEncoderSetIndexBuffer: function(passId, bufferId, offset, size) {
+  wgpuRenderPassEncoderSetIndexBuffer: function(passId, bufferId, format, {{{ defineI64Param('offset') }}}, size) {
+    {{{ receiveI64ParamAsI32s('offset') }}}
+    var offset = {{{ gpu.makeU64ToNumber('offset_low', 'offset_high') }}};
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
     var buffer = WebGPU.mgrBuffer.get(bufferId);
-    pass["setIndexBuffer"](buffer, offset, size);
+    pass["setIndexBuffer"](buffer, WebGPU.IndexFormat[format], offset, size);
+  },
+  wgpuRenderPassEncoderSetIndexBufferWithFormat: function(passId, bufferId, format, {{{ defineI64Param('offset') }}}, size) {
+    {{{ receiveI64ParamAsI32s('offset') }}}
+    var offset = {{{ gpu.makeU64ToNumber('offset_low', 'offset_high') }}};
+    var pass = WebGPU.mgrRenderPassEncoder.get(passId);
+    var buffer = WebGPU.mgrBuffer.get(bufferId);
+    pass["setIndexBuffer"](buffer, WebGPU.IndexFormat[format], offset, size);
   },
   wgpuRenderPassEncoderSetPipeline: function(passId, pipelineId) {
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
@@ -1630,12 +1754,19 @@ var LibraryWebGPU = {
       pass["setBindGroup"](groupIndex, group, offsets);
     }
   },
-  wgpuRenderBundleEncoderSetIndexBuffer: function(bundleId, bufferId, {{{ defineI64Param('offset') }}}) {
+  wgpuRenderBundleEncoderSetIndexBuffer: function(bundleId, bufferId, format, {{{ defineI64Param('offset') }}}, size) {
     {{{ receiveI64ParamAsI32s('offset') }}}
     var offset = {{{ gpu.makeU64ToNumber('offset_low', 'offset_high') }}};
     var pass = WebGPU.mgrRenderBundleEncoder.get(bundleId);
     var buffer = WebGPU.mgrBuffer.get(bufferId);
-    pass["setIndexBuffer"](buffer, offset);
+    pass["setIndexBuffer"](buffer, WebGPU.IndexFormat[format], offset, size);
+  },
+  wgpuRenderBundleEncoderSetIndexBufferWithFormat: function(bundleId, bufferId, format, {{{ defineI64Param('offset') }}}, size) {
+    {{{ receiveI64ParamAsI32s('offset') }}}
+    var offset = {{{ gpu.makeU64ToNumber('offset_low', 'offset_high') }}};
+    var pass = WebGPU.mgrRenderBundleEncoder.get(bundleId);
+    var buffer = WebGPU.mgrBuffer.get(bufferId);
+    pass["setIndexBuffer"](buffer, WebGPU.IndexFormat[format], offset, size);
   },
   wgpuRenderBundleEncoderSetPipeline: function(bundleId, pipelineId) {
     var pass = WebGPU.mgrRenderBundleEncoder.get(bundleId);
