@@ -367,9 +367,6 @@ i16x8 TESTFN i16x8_abs(i16x8 vec) {
 i16x8 TESTFN i16x8_neg(i16x8 vec) {
   return -vec;
 }
-int32_t TESTFN i16x8_any_true(i16x8 vec) {
-  return __builtin_wasm_any_true_i16x8(vec);
-}
 int32_t TESTFN i16x8_all_true(i16x8 vec) {
   return __builtin_wasm_all_true_i16x8(vec);
 }
@@ -413,9 +410,6 @@ i32x4 TESTFN i32x4_abs(i32x4 vec) {
 i32x4 TESTFN i32x4_neg(i32x4 vec) {
   return -vec;
 }
-int32_t TESTFN i32x4_any_true(i32x4 vec) {
-  return __builtin_wasm_any_true_i32x4(vec);
-}
 int32_t TESTFN i32x4_all_true(i32x4 vec) {
   return __builtin_wasm_all_true_i32x4(vec);
 }
@@ -441,14 +435,6 @@ i32x4 TESTFN i32x4_mul(i32x4 x, i32x4 y) {
 i64x2 TESTFN i64x2_neg(i64x2 vec) {
   return -vec;
 }
-#ifdef __wasm_unimplemented_simd128__
-int32_t TESTFN i64x2_any_true(i64x2 vec) {
-  return __builtin_wasm_any_true_i64x2(vec);
-}
-int32_t TESTFN i64x2_all_true(i64x2 vec) {
-  return __builtin_wasm_all_true_i64x2(vec);
-}
-#endif // __wasm_unimplemented_simd128__
 i64x2 TESTFN i64x2_shl(i64x2 vec, int32_t shift) {
   return vec << shift;
 }
@@ -1140,10 +1126,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
     i16x8_neg((i16x8){0, 1, 42, -3, -56, 32767, -32768, 32766}),
     ((i16x8){0, -1, -42, 3, 56, -32767, -32768, -32766})
   );
-  expect_eq(i16x8_any_true((i16x8){0, 0, 0, 0, 0, 0, 0, 0}), 0);
-  expect_eq(i16x8_any_true((i16x8){0, 0, 1, 0, 0, 0, 0, 0}), 1);
-  expect_eq(i16x8_any_true((i16x8){1, 1, 1, 1, 1, 0, 1, 1}), 1);
-  expect_eq(i16x8_any_true((i16x8){1, 1, 1, 1, 1, 1, 1, 1}), 1);
   expect_eq(i16x8_all_true((i16x8){0, 0, 0, 0, 0, 0, 0, 0}), 0);
   expect_eq(i16x8_all_true((i16x8){0, 0, 1, 0, 0, 0, 0, 0}), 0);
   expect_eq(i16x8_all_true((i16x8){1, 1, 1, 1, 1, 0, 1, 1}), 0);
@@ -1232,10 +1214,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
   // i32x4 arithmetic
   expect_vec(i32x4_abs((i32x4){0, 1, 0x80000000, 0x80000001}), ((i32x4){0, 1, 0x80000000, 0x7fffffff}));
   expect_vec(i32x4_neg((i32x4){0, 1, 0x80000000, 0x80000001}), ((i32x4){0, -1, 0x80000000, 0x7fffffff}));
-  expect_eq(i32x4_any_true((i32x4){0, 0, 0, 0}), 0);
-  expect_eq(i32x4_any_true((i32x4){0, 0, 1, 0}), 1);
-  expect_eq(i32x4_any_true((i32x4){1, 0, 1, 1}), 1);
-  expect_eq(i32x4_any_true((i32x4){1, 1, 1, 1}), 1);
   expect_eq(i32x4_all_true((i32x4){0, 0, 0, 0}), 0);
   expect_eq(i32x4_all_true((i32x4){0, 0, 1, 0}), 0);
   expect_eq(i32x4_all_true((i32x4){1, 0, 1, 1}), 0);
@@ -1253,14 +1231,6 @@ int EMSCRIPTEN_KEEPALIVE __attribute__((__optnone__)) main(int argc, char** argv
 
   // i64x2 arithmetic
   expect_vec(i64x2_neg((i64x2){0x8000000000000000, 42}), ((i64x2){0x8000000000000000, -42}));
-#ifdef __wasm_unimplemented_simd128__
-  expect_eq(i64x2_any_true((i64x2){0, 0}), 0);
-  expect_eq(i64x2_any_true((i64x2){1, 0}), 1);
-  expect_eq(i64x2_any_true((i64x2){1, 1}), 1);
-  expect_eq(i64x2_all_true((i64x2){0, 0}), 0);
-  expect_eq(i64x2_all_true((i64x2){1, 0}), 0);
-  expect_eq(i64x2_all_true((i64x2){1, 1}), 1);
-#endif // __wasm_unimplemented_simd128__
   expect_vec(i64x2_shl((i64x2){1, 0x8000000000000000}, 1), ((i64x2){2, 0}));
   expect_vec(i64x2_shl((i64x2){1, 0x8000000000000000}, 64), ((i64x2){1, 0x8000000000000000}));
   expect_vec(i64x2_shr_s((i64x2){1, 0x8000000000000000}, 1), ((i64x2){0, 0xc000000000000000}));
