@@ -3,7 +3,6 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-from __future__ import print_function
 import os
 import shutil
 import tempfile
@@ -55,8 +54,8 @@ def try_delete(pathname):
 
 
 class TempFiles(object):
-  def __init__(self, tmp, save_debug_files=False):
-    self.tmp = tmp
+  def __init__(self, tmpdir, save_debug_files):
+    self.tmpdir = tmpdir
     self.save_debug_files = save_debug_files
     self.to_clean = []
 
@@ -67,17 +66,19 @@ class TempFiles(object):
 
   def get(self, suffix):
     """Returns a named temp file with the given prefix."""
-    named_file = tempfile.NamedTemporaryFile(dir=self.tmp, suffix=suffix, delete=False)
+    named_file = tempfile.NamedTemporaryFile(dir=self.tmpdir, suffix=suffix, delete=False)
     self.note(named_file.name)
     return named_file
 
   def get_file(self, suffix):
-    """Returns an object representing a RAII-like access to a temp file, that has convenient pythonesque
-    semantics for being used via a construct 'with TempFiles.get_file(..) as filename:'. The file will be
-    deleted immediately once the 'with' block is exited."""
+    """Returns an object representing a RAII-like access to a temp file
+    that has convenient pythonesque semantics for being used via a construct
+      'with TempFiles.get_file(..) as filename:'.
+    The file will be deleted immediately once the 'with' block is exited.
+    """
     class TempFileObject(object):
       def __enter__(self_):
-        self_.file = tempfile.NamedTemporaryFile(dir=self.tmp, suffix=suffix, delete=False)
+        self_.file = tempfile.NamedTemporaryFile(dir=self.tmpdir, suffix=suffix, delete=False)
         self_.file.close() # NamedTemporaryFile passes out open file handles, but callers prefer filenames (and open their own handles manually if needed)
         return self_.file.name
 
@@ -88,20 +89,14 @@ class TempFiles(object):
 
   def get_dir(self):
     """Returns a named temp directory with the given prefix."""
-    directory = tempfile.mkdtemp(dir=self.tmp)
+    directory = tempfile.mkdtemp(dir=self.tmpdir)
     self.note(directory)
     return directory
 
   def clean(self):
     if self.save_debug_files:
-      print('not cleaning up temp files since in debug-save mode, see them in %s' % (self.tmp,), file=sys.stderr)
+      print(f'not cleaning up temp files since in debug-save mode, see them in {self.tmpdir}', file=sys.stderr)
       return
     for filename in self.to_clean:
       try_delete(filename)
     self.to_clean = []
-
-  def run_and_clean(self, func):
-    try:
-      return func()
-    finally:
-      self.clean()
